"""This module provides CSS support for UI"""

from gi.repository import Gtk
from . import settings

def get_css_provider() -> Gtk.CssProvider:
    """Create css provider from stylesheet file and return it"""
    cssfile = open(settings.workdir + '/styles.css', 'rb')
    cssdata = cssfile.read()
    cssfile.close()

    cssprovider = Gtk.CssProvider()
    cssprovider.load_from_data(cssdata)

    return cssprovider



import unittest
import linecache
import shutil
from importlib import reload

import os
os.environ['XDG_CONFIG_HOME'] = os.path.dirname(os.path.abspath(__file__)) + '/settings/default'
from tlpui import settings


class MyTestCase(unittest.TestCase):
    def test_tlp_settings_broken(self):
        os.environ['XDG_CONFIG_HOME'] = os.path.dirname(os.path.abspath(__file__)) + '/settings/default'
        reload(settings)
        settings.userconfigfile.write_text("[default]")
        reload(settings)
        self.assertEqual(linecache.getline(str(settings.userconfigfile), 1), "[default]\n")
        self.assertEqual(linecache.getline(str(settings.userconfigfile), 2), "language = en_EN\n")
        self.assertEqual(linecache.getline(str(settings.userconfigfile), 3), "activecategorie = 0\n")
        self.assertEqual(linecache.getline(str(settings.userconfigfile), 4), "windowxsize = 900\n")
        self.assertEqual(linecache.getline(str(settings.userconfigfile), 5), "windowysize = 600\n")


    def test_tlp_settings_creation(self):
        newsettingspath = os.path.dirname(os.path.abspath(__file__)) + '/settings/creation'
        os.environ['XDG_CONFIG_HOME'] = newsettingspath
        if os.path.exists(newsettingspath):
            shutil.rmtree(newsettingspath)
        reload(settings)
        self.assertEqual(linecache.getline(str(settings.userconfigfile), 1), "[default]\n")
        self.assertEqual(linecache.getline(str(settings.userconfigfile), 2), "language = en_EN\n")
        self.assertEqual(linecache.getline(str(settings.userconfigfile), 3), "activecategorie = 0\n")
        self.assertEqual(linecache.getline(str(settings.userconfigfile), 4), "windowxsize = 900\n")
        self.assertEqual(linecache.getline(str(settings.userconfigfile), 5), "windowysize = 600\n")


if __name__ == '__main__':
    unittest.main()



from setuptools import setup

setup(
    name="TLPUI",
    version="0.1",
    author="Daniel Christophis",
    author_email="code@devmind.org",
    description="GTK UI for tlp (Python 3)",
    url="https://github.com/d4nj1/TLPUI",
    license="GPLv2",
    packages=["tlpui", "tlpui.ui_config_objects"],
    package_data={'tlpui': [
        'styles.css',
        'configschema/*.json',
        'defaults/*.conf',
        'icons/*.svg',
        'icons/themeable/hicolor/scalable/actions/*.svg',
        'lang/*/LC_MESSAGES/*.mo'
    ]},
    entry_points={
        'gui_scripts': [
            'tlpui = tlpui.__main__:main'
        ]
    }
)



import gi

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk

from collections import OrderedDict

from . import settings
from . import language
from .ui_config_objects import gtkswitch, gtkentry, gtkselection, gtkmultiselection, gtkcheckbutton, gtkspinbutton, gtktoggle, gtkusblist, gtkpcilist, gtkdisklist, gtkdisklistview
from .file import ConfType, TlpConfig, get_json_schema_object
from .uihelper import get_theme_image, StateImage, EXPECTED_ITEM_MISSING_TEXT


def store_page_num(self, page, page_num):
    settings.activecategorie = page_num


def create_config_box(window) -> Gtk.Box:
    notebook = Gtk.Notebook()
    notebook.set_name('configNotebook')
    notebook.set_tab_pos(Gtk.PositionType.LEFT)

    categories = get_json_schema_object('categories')
    tlp_categories = get_tlp_categories(window, categories)
    for name, categorydata in tlp_categories.items():
        categorylabel = Gtk.Label(categorydata[0])
        categorylabel.set_alignment(1, 0.5)
        categorylabel.set_margin_top(6)
        categorylabel.set_margin_bottom(6)
        categorylabel.set_margin_left(6)
        categorylabel.set_margin_right(6)

        viewport = Gtk.Viewport()
        viewport.set_name('categoryViewport')
        viewport.add(categorydata[1])

        scroll = Gtk.ScrolledWindow()
        scroll.add(viewport)

        categoryimage = get_theme_image('tlpui-{}-symbolic'.format(name), Gtk.IconSize.MENU)

        labelbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=0)
        labelbox.pack_start(categoryimage, False, False, 0)
        labelbox.pack_start(categorylabel, True, True, 0)
        labelbox.show_all()

        notebook.append_page(scroll, labelbox)

    notebook.connect('switch-page', store_page_num)

    containerbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
    containerbox.pack_start(notebook, True, True, 0)

    activecategorie = settings.activecategorie
    notebook.show_all()
    notebook.set_current_page(activecategorie)

    return containerbox


def create_config_widget(configname: str, objecttype: str, objectvalues: str, window: Gtk.Window) -> Gtk.Widget:
    configwidget = Gtk.Widget

    if (objecttype == 'entry'):
        configwidget = gtkentry.create_entry(configname)
    elif (objecttype == 'usblist'):
        configwidget = gtkusblist.create_list(configname, window)
    elif (objecttype == 'pcilist'):
        configwidget = gtkpcilist.create_list(configname, window)
    elif (objecttype == 'disklist'):
        configwidget = gtkdisklist.create_list(configname, window)
    elif (objecttype == 'disklistview'):
        configwidget = gtkdisklistview.create_view(configname)
    elif (objecttype == 'bselect'):
        configwidget = gtkswitch.create_state_switch(configname, objectvalues)
    elif (objecttype == 'select'):
        configwidget = gtkselection.create_selection_box(configname, objectvalues)
    elif (objecttype == 'multiselect'):
        configwidget = gtkmultiselection.create_multi_selection_box(configname, objectvalues)
    elif (objecttype == 'check'):
        configwidget = gtkcheckbutton.create_checkbutton_box(configname, objectvalues)
    elif (objecttype == 'numeric'):
        configwidget = gtkspinbutton.create_numeric_spinbutton(configname, objectvalues)

    return configwidget


def get_state_image(configname: str):
    image = Gtk.Image()
    defaultvalue = settings.tlpconfig_defaults[configname].get_value()
    defaultstate = settings.tlpconfig_defaults[configname].is_enabled()
    settings.imagestate[configname] = StateImage(defaultvalue, defaultstate, image)
    settings.tlpconfig[configname].refresh_image_state()
    return image


def get_type_image(configname: str) -> Gtk.Image:
    tlpconfig = settings.tlpconfig[configname]      # type: TlpConfig
    conftype = tlpconfig.get_conf_type()
    conftypeimage = Gtk.Image()

    if conftype == ConfType.DROPIN:
        conftypeimage = Gtk.Image.new_from_file(settings.icondir + 'dropin.svg')
        conftypeimage.set_tooltip_text(tlpconfig.get_conf_path())
    elif conftype == ConfType.USER:
        conftypeimage = Gtk.Image.new_from_file(settings.icondir + 'user.svg')
        conftypeimage.set_tooltip_text(tlpconfig.get_conf_path())

    return conftypeimage


def create_item_box(configobjects, doc, grouptitle, window) -> Gtk.Box:
    configuibox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)

    if len(configobjects) > 1:
        transgrouptitle = grouptitle + "__GROUP_TITLE"
        grouplabel = Gtk.Label()
        grouplabel.set_markup(' <b>{}</b> '.format(language.CDT_(transgrouptitle)))
        grouplabel.set_use_markup(True)
        grouplabel.set_margin_bottom(12)
        grouplabel.set_halign(Gtk.Align.START)
        grouplabel.set_valign(Gtk.Align.START)

        configuibox.pack_start(grouplabel, False, False, 0)

    for configobject in configobjects:
        configname = configobject[0]
        configtype = configobject[1]
        possiblevalues = configobject[2]

        tlpuiobject = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=18)
        tlpuiobject.set_margin_start(18)
        tlpuiobject.set_margin_end(18)

        if configname not in settings.tlpconfig.keys():
            missingcheckbox = Gtk.CheckButton()
            missingcheckbox.set_child_visible(False)
            missingstatetogglebox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
            missingstatetogglebox.pack_start(missingcheckbox, False, False, 0)
            missingstatetogglebox.set_halign(Gtk.Align.CENTER)
            missingstatetogglebox.set_valign(Gtk.Align.CENTER)

            missingconfiglabel = Gtk.Label(xalign=0)
            missingconfiglabel.set_name('missingConfigLabel')
            missingconfiglabel.set_markup(' <b>{}</b> - <i>{}</i> '.format(configname, EXPECTED_ITEM_MISSING_TEXT))
            missingconfiglabel.set_use_markup(True)

            tlpuiobject.pack_start(missingstatetogglebox, False, False, 0)
            tlpuiobject.pack_start(missingconfiglabel, False, False, 0)
            configuibox.pack_start(tlpuiobject, True, True, 0)
            continue

        statetogglebox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
        tlpconfigbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=0)

        # on/off state
        toggle = gtktoggle.create_toggle_button(configname, tlpconfigbox)
        statetogglebox.pack_start(toggle, False, False, 0)
        statetogglebox.set_halign(Gtk.Align.CENTER)
        statetogglebox.set_valign(Gtk.Align.CENTER)

        # config state image
        configstateimage = get_state_image(configname)

        # config type image
        configtypeimage = get_type_image(configname)

        # specific config gtk object
        configwidget = create_config_widget(configname, configtype, possiblevalues, window)
        configwidget.set_margin_top(6)
        configwidget.set_margin_bottom(6)
        configwidget.set_margin_left(6)

        # object label
        transconfigtitle = configname + "__ID_TITLE"
        configlabel = Gtk.Label(xalign=0)
        configlabel.set_markup(' <b>{}</b> '.format(language.CDT_(transconfigtitle)))
        configlabel.set_use_markup(True)
        configlabel.set_size_request(300, 0)

        # combine boxes
        tlpconfigbox.pack_start(configlabel, False, False, 0)
        tlpconfigbox.pack_start(configwidget, True, True, 0)

        if configname.endswith('_BAT'):
            image = Gtk.Image.new_from_file(settings.icondir + 'OnBAT.svg')
            tlpconfigbox.pack_start(image, False, False, 12)
        elif configname.endswith('_AC'):
            image = Gtk.Image.new_from_file(settings.icondir + 'OnAC.svg')
            tlpconfigbox.pack_start(image, False, False, 12)

        tlpuiobject.pack_start(statetogglebox, False, False, 0)
        tlpuiobject.pack_start(tlpconfigbox, False, False, 0)
        tlpuiobject.pack_start(configstateimage, False, False, 0)
        tlpuiobject.pack_end(configtypeimage, False, False, 0)

        configuibox.pack_start(tlpuiobject, True, True, 0)

    # object description
    configdescriptionlabel = Gtk.Label()
    configdescriptionlabel.set_markup(doc)
    configdescriptionlabel.set_line_wrap(True)
    configdescriptionlabel.set_margin_top(6)
    configdescriptionlabel.set_margin_bottom(12)
    configdescriptionlabel.set_margin_start(48)
    configdescriptionlabel.set_halign(Gtk.Align.START)
    configdescriptionlabel.set_valign(Gtk.Align.START)

    # horizontal separator
    hseparator = Gtk.HSeparator()

    configuibox.pack_start(configdescriptionlabel, True, True, 0)
    configuibox.pack_start(hseparator, True, True, 0)

    return configuibox


def get_tlp_categories(window, categories) -> OrderedDict:
    propertyobjects = OrderedDict()

    for category in categories:
        categorybox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=12)

        configs = category['configs']
        for config in configs:
            grouptitle = ""
            configobjects = list()

            if 'group' in config:
                grouptitle = config['group']
                transdescription = grouptitle + "__GROUP_DESCRIPTION"
                groupitems = config['ids']
                for groupitem in groupitems:
                    id = groupitem['id']
                    type = groupitem['type']
                    values = groupitem['values']
                    configobjects.append([id, type, values])
            else:
                id = config['id']
                transdescription = id + "__ID_DESCRIPTION"
                type = config['type']
                values = config['values']
                configobjects.append([id, type, values])

            description = language.CDT_(transdescription)

            configbox = create_item_box(configobjects, description, grouptitle, window)
            configbox.set_margin_start(12)
            configbox.set_margin_end(12)
            configbox.set_margin_top(12)
            categorybox.pack_start(configbox, False, False, 0)

        categoryname = category['name']
        transcategory = categoryname + "__CATEGORY_TITLE"
        categorylabel = language.CDT_(transcategory)
        propertyobjects[categoryname] = [categorylabel, categorybox]

    return propertyobjects



import sys
from gi.repository import Gtk

from subprocess import check_output
from collections import OrderedDict
from ..uihelper import get_theme_image
from .. import mainui
from .. import file
from .. import settings


def create_list(configname: str, window: Gtk.Window) -> Gtk.Box:
    tlpobject = settings.tlpconfig[configname]
    box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
    label = Gtk.Label(tlpobject.get_value())
    label.set_width_chars(len(tlpobject.get_value())+5)

    button = Gtk.Button(label=' Edit', image=get_theme_image('edit-symbolic', Gtk.IconSize.BUTTON))
    button.connect('clicked', edit_list, window)
    button.set_always_show_image(True)

    box.pack_start(label, False, False, 0)
    box.pack_start(button, False, False, 12)
    return box


def edit_list(self: Gtk.Button, window: Gtk.Window):
    notebook = Gtk.Notebook()
    notebook.set_tab_pos(Gtk.PositionType.TOP)

    configranges = get_disk_config_ranges()
    existingdisks = read_existing_disk_config()

    disks = OrderedDict()
    keeps = OrderedDict()
    disklist = check_output(["tlp", "diskid"]).decode(sys.stdout.encoding)
    for line in disklist.splitlines():
        diskid = line.split(':')[0].lstrip().rstrip()

        diskvalues = get_disk_values(diskid, existingdisks)
        defaultvalues = diskvalues[0]
        keepvalues = diskvalues[1]

        notebookgrid = Gtk.Grid()
        notebookgrid.set_row_homogeneous(True)
        notebookgrid.set_column_spacing(12)

        configlabel = Gtk.Label(label='Config', halign=Gtk.Align.START)
        valuelabel = Gtk.Label(label='Value', halign=Gtk.Align.START)
        keeplabel = Gtk.Label(label='Hardware default', halign=Gtk.Align.START)

        apmlevelonaclabel = Gtk.Label(label='DISK_APM_LEVEL_ON_AC', halign=Gtk.Align.START)
        apmlevelonacspiner = create_spinbutton(configranges[0], int(defaultvalues[0]))
        apmlevelonackeep = create_keep(apmlevelonacspiner, keepvalues[0])

        apmlevelonbatlabel = Gtk.Label(label='DISK_APM_LEVEL_ON_BAT', halign=Gtk.Align.START)
        apmlevelonbatspiner = create_spinbutton(configranges[1], int(defaultvalues[1]))
        apmlevelonbatkeep = create_keep(apmlevelonbatspiner, keepvalues[1])

        spindowntimeoutonaclabel = Gtk.Label(label='DISK_SPINDOWN_TIMEOUT_ON_AC', halign=Gtk.Align.START)
        spindowntimeoutonacspiner = create_spinbutton(configranges[2], int(defaultvalues[2]))
        spindowntimeoutonackeep = create_keep(spindowntimeoutonacspiner, keepvalues[2])

        spindowntimeoutonbatlabel = Gtk.Label(label='DISK_SPINDOWN_TIMEOUT_ON_BAT', halign=Gtk.Align.START)
        spindowntimeoutonbatspiner = create_spinbutton(configranges[3], int(defaultvalues[3]))
        spindowntimeoutonbatkeep = create_keep(spindowntimeoutonbatspiner, keepvalues[3])

        ioschedlabel = Gtk.Label(label='DISK_IOSCHED', halign=Gtk.Align.START)
        ioschedselect = create_selectbox(configranges[4], defaultvalues[4])
        ioschedkeep = create_keep(ioschedselect, keepvalues[4])

        notebookgrid.attach(Gtk.Label(' '), 0, 0, 1, 1)
        notebookgrid.attach(configlabel, 1, 0, 1, 1)
        notebookgrid.attach(valuelabel, 2, 0, 1, 1)
        notebookgrid.attach(keeplabel, 3, 0, 1, 1)
        notebookgrid.attach(Gtk.Label(' '), 4, 0, 1, 1)

        notebookgrid.attach(apmlevelonaclabel, 1, 1, 1, 1)
        notebookgrid.attach(apmlevelonacspiner, 2, 1, 1, 1)
        notebookgrid.attach(apmlevelonackeep, 3, 1, 1, 1)

        notebookgrid.attach(apmlevelonbatlabel, 1, 2, 1, 1)
        notebookgrid.attach(apmlevelonbatspiner, 2, 2, 1, 1)
        notebookgrid.attach(apmlevelonbatkeep, 3, 2, 1, 1)

        notebookgrid.attach(spindowntimeoutonaclabel, 1, 3, 1, 1)
        notebookgrid.attach(spindowntimeoutonacspiner, 2, 3, 1, 1)
        notebookgrid.attach(spindowntimeoutonackeep, 3, 3, 1, 1)

        notebookgrid.attach(spindowntimeoutonbatlabel, 1, 4, 1, 1)
        notebookgrid.attach(spindowntimeoutonbatspiner, 2, 4, 1, 1)
        notebookgrid.attach(spindowntimeoutonbatkeep, 3, 4, 1, 1)

        notebookgrid.attach(ioschedlabel, 1, 5, 1, 1)
        notebookgrid.attach(ioschedselect, 2, 5, 1, 1)
        notebookgrid.attach(ioschedkeep, 3, 5, 1, 1)

        notebooklabel = Gtk.Label(diskid)
        notebook.append_page(notebookgrid, notebooklabel)

        disks[diskid] = [apmlevelonacspiner, apmlevelonbatspiner, spindowntimeoutonacspiner, spindowntimeoutonbatspiner, ioschedselect]
        keeps[diskid] = [apmlevelonackeep, apmlevelonbatkeep, spindowntimeoutonackeep, spindowntimeoutonbatkeep, ioschedkeep]

    dialog = Gtk.Dialog('Disk devices', window, 0, (
        Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
        Gtk.STOCK_OK, Gtk.ResponseType.OK
    ))

    contentarea = dialog.get_content_area()
    contentarea.set_spacing(6)
    contentarea.add(notebook)

    dialog.show_all()

    response = dialog.run()
    if response == Gtk.ResponseType.OK:
        diskdevices = ''
        apmlevelonac = ''
        apmlevelonbat = ''
        spindowntimeoutonac = ''
        spindowntimeoutonbat = ''
        iosched = ''

        for key, value in disks.items():
            diskdevices = diskdevices + ' ' + key
            keep = keeps[key]
            apmlevelonac = apmlevelonac + ' ' + ('keep' if keep[0].get_active() else str(int(value[0].get_value())))
            apmlevelonbat = apmlevelonbat + ' ' + ('keep' if keep[1].get_active() else str(int(value[1].get_value())))
            spindowntimeoutonac = spindowntimeoutonac + ' ' + ('keep' if keep[2].get_active() else str(int(value[2].get_value())))
            spindowntimeoutonbat = spindowntimeoutonbat + ' ' + ('keep' if keep[3].get_active() else str(int(value[3].get_value())))
            iosched = iosched + ' ' + ('keep' if keep[4].get_active() else value[4].get_active_text())

        set_tlp_value('DISK_DEVICES', diskdevices.lstrip())
        set_tlp_value('DISK_APM_LEVEL_ON_AC', apmlevelonac.lstrip())
        set_tlp_value('DISK_APM_LEVEL_ON_BAT', apmlevelonbat.lstrip())
        set_tlp_value('DISK_SPINDOWN_TIMEOUT_ON_AC', spindowntimeoutonac.lstrip())
        set_tlp_value('DISK_SPINDOWN_TIMEOUT_ON_BAT', spindowntimeoutonbat.lstrip())
        set_tlp_value('DISK_IOSCHED', iosched.lstrip())

        mainui.load_tlp_config(self, window, False)

    dialog.destroy()


def set_tlp_value(configname: str, value: str):
    settings.tlpconfig[configname].set_value(value)


def get_disk_values(diskid: str, existingdisks: OrderedDict):
    defaultvalues = ['254', '128', '0', '0', 'cfq']
    keepvalues = [False, False, False, False, False]
    if diskid in existingdisks.keys():
        existingvalues = existingdisks[diskid]
        for index in range(0, 5):
            if existingvalues[index] == 'keep':
                keepvalues[index] = True
            else:
                defaultvalues[index] = existingvalues[index]
    return defaultvalues, keepvalues


def create_keep(gtkwidget: Gtk.Widget, isactive: bool):
    checkbutton = Gtk.CheckButton(halign=Gtk.Align.CENTER)
    if isactive:
        checkbutton.set_active(True)
        gtkwidget.set_opacity(0.3)

    checkbutton.connect('toggled', on_button_toggled, gtkwidget)
    return checkbutton


def on_button_toggled(self: Gtk.CheckButton, gtkwidget: Gtk.Widget):
    if self.get_active():
        gtkwidget.set_opacity(0.3)
    else:
        gtkwidget.set_opacity(1)


def create_spinbutton(values: str, configvalue: str):
    range = values.split('-')
    adjustment = Gtk.Adjustment(0, float(range[0]), float(range[1]), 1, 10, 0)

    spinbutton = Gtk.SpinButton()
    spinbutton.set_numeric(True)
    spinbutton.set_adjustment(adjustment)
    spinbutton.set_value(float(configvalue))
    return spinbutton


def create_selectbox(values: str, configvalue: str):
    combobox = Gtk.ComboBoxText()
    selectitems = values.split(',')

    countid = 0
    selectid = 0

    for item in selectitems:
        combobox.append_text(item)
        if item in configvalue:
            selectid = countid
        countid += 1

    combobox.set_active(selectid)
    return combobox


def read_existing_disk_config() -> OrderedDict:
    devices = settings.tlpconfig['DISK_DEVICES'].get_value().split(' ')
    apmlevelonac = settings.tlpconfig['DISK_APM_LEVEL_ON_AC'].get_value().split(' ')
    apmlevelonbat = settings.tlpconfig['DISK_APM_LEVEL_ON_BAT'].get_value().split(' ')
    spindowntimeoutonac = settings.tlpconfig['DISK_SPINDOWN_TIMEOUT_ON_AC'].get_value().split(' ')
    spindowntimeoutonbat = settings.tlpconfig['DISK_SPINDOWN_TIMEOUT_ON_BAT'].get_value().split(' ')
    iosched = settings.tlpconfig['DISK_IOSCHED'].get_value().split(' ')

    existingdiskconfig = OrderedDict()
    index = 0
    for device in devices:
        existingdiskconfig[device] = [apmlevelonac[index], apmlevelonbat[index], spindowntimeoutonac[index], spindowntimeoutonbat[index], iosched[index]]
        index += 1

    return existingdiskconfig


def get_disk_config_ranges():
    apmlevelrangeac = ''
    apmlevelrangebat = ''
    spindownrangetimeoutac = ''
    spindownrangetimeoutbat = ''
    ioschedvalues = ''

    categories = file.get_json_schema_object('categories')
    for category in categories:
        if category['name'] != 'Disks':
            continue

        for config in category['configs']:
            if 'group' in config:
                if config['group'] == 'DISK_APM_LEVEL':
                    for item in config['ids']:
                        if item['id'] == 'DISK_APM_LEVEL_ON_AC':
                            apmlevelrangeac = item['values']
                        elif item['id'] == 'DISK_APM_LEVEL_ON_BAT':
                            apmlevelrangebat = item['values']
                elif config['group'] == 'DISK_SPINDOWN_TIMEOUT':
                    for item in config['ids']:
                        if item['id'] == 'DISK_SPINDOWN_TIMEOUT_ON_AC':
                            spindownrangetimeoutac = item['values']
                        elif item['id'] == 'DISK_SPINDOWN_TIMEOUT_ON_BAT':
                            spindownrangetimeoutbat = item['values']
            else:
                if config['id'] == 'DISK_IOSCHED':
                    ioschedvalues = config['values']

    return [apmlevelrangeac, apmlevelrangebat, spindownrangetimeoutac, spindownrangetimeoutbat, ioschedvalues]



"""This module provides helper functions for TLPUI"""

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
from shutil import which
from . import language
from . import settings


EXPECTED_ITEM_MISSING_TEXT = language.UH_('Expected item missing in config file')  # type: str
SUDO_MISSING_TEXT = language.UH_('Install pkexec, gksu, gksudo, kdesu or kdesudo first.')  # type: str
DEFAULT_STATE_TEXT = language.UH_('Default state:')  # type: str
DEFAULT_VALUE_TEXT = language.UH_('Default value:')  # type: str
CHANGED_STATE_TEXT = language.UH_('CHANGED')  # type: str


def get_graphical_sudo() -> str:
    """Fetch available graphical sudo command"""
    sudo = which("pkexec")
    if sudo is None:
        sudo = which("gksu")
    if sudo is None:
        sudo = which("gksudo")
    if sudo is None:
        sudo = which("kdesu")
    if sudo is None:
        sudo = which("kdesudo")
    return sudo


def get_theme_image(iconname: str, iconsize: Gtk.IconSize) -> Gtk.Image:
    """Fetch image from theme or return fallback if missing"""
    if Gtk.IconTheme.get_default().has_icon(iconname):
        return Gtk.Image().new_from_icon_name(iconname, iconsize)
    else:
        return Gtk.Image().new_from_file(settings.icondir + 'themeable/hicolor/scalable/actions/' + iconname + '.svg')


class StateImage:
    """Class to display configuration item state"""
    def __init__(self, defaultvalue, defaultstate, stateimage: Gtk.Image):
        self.defaultvalue = str(defaultvalue)
        self.defaultstate = bool(defaultstate)
        self.stateimage = stateimage

    def refresh_image_state(self, value: str, store: str, enabled: bool, enabledstore: bool) -> None:
        """Refresh image and description by changed state"""
        changed = False
        if enabled != enabledstore or value != store:
            changed = True

        enabledtext = ''
        if enabled != self.defaultstate:
            enabledtext += '{} {}'.format(DEFAULT_STATE_TEXT, str(self.defaultstate))

        if value == self.defaultvalue:
            if not changed and enabledtext == '':
                self.stateimage.clear()
            elif not changed and enabledtext != '':
                self.stateimage.set_from_icon_name(Gtk.STOCK_INFO, Gtk.IconSize.BUTTON)
                self.stateimage.set_tooltip_text('{}'.format(enabledtext))
            elif changed and enabledtext == '':
                self.stateimage.set_from_icon_name(Gtk.STOCK_UNDO, Gtk.IconSize.BUTTON)
                self.stateimage.set_tooltip_text('{}'.format(CHANGED_STATE_TEXT))
            elif changed and enabledtext != '':
                self.stateimage.set_from_icon_name(Gtk.STOCK_UNDO, Gtk.IconSize.BUTTON)
                self.stateimage.set_tooltip_text('{}\n{}'.format(CHANGED_STATE_TEXT, enabledtext))
        else:
            defaulttext = '{} {}'.format(DEFAULT_VALUE_TEXT, self.defaultvalue)
            if not changed and enabledtext == '':
                self.stateimage.set_from_icon_name(Gtk.STOCK_INFO, Gtk.IconSize.BUTTON)
                self.stateimage.set_tooltip_text(defaulttext)
            elif not changed and enabledtext != '':
                self.stateimage.set_from_icon_name(Gtk.STOCK_INFO, Gtk.IconSize.BUTTON)
                self.stateimage.set_tooltip_text('{}\n{}'.format(enabledtext, defaulttext))
            elif changed and enabledtext == '':
                self.stateimage.set_from_icon_name(Gtk.STOCK_UNDO, Gtk.IconSize.BUTTON)
                self.stateimage.set_tooltip_text('{}\n{}'.format(CHANGED_STATE_TEXT, defaulttext))
            elif changed and enabledtext != '':
                self.stateimage.set_from_icon_name(Gtk.STOCK_UNDO, Gtk.IconSize.BUTTON)
                self.stateimage.set_tooltip_text('{}\n{}\n{}'.format(CHANGED_STATE_TEXT, enabledtext, defaulttext))



import configparser
import re
import sys
from subprocess import check_output
from os import path, getenv
from pathlib import Path

# application folder settings
workdir = path.dirname(path.abspath(__file__))
langdir = workdir + '/lang/'
icondir = workdir + '/icons/'

# default user params
language = 'en_EN'
activecategorie = 0
windowxsize = 900
windowysize = 600

# user config
userconfighome = getenv("XDG_CONFIG_HOME", "")
if userconfighome == "":
    userconfigpath = Path(str(Path.home()) + "/.config/tlpui")
else:
    userconfigpath = Path(str(userconfighome) + "/tlpui")
userconfigfile = Path(str(userconfigpath) + "/tlpui.cfg")


def persist():
    config = configparser.ConfigParser()
    config['default'] = {}
    config['default']['language'] = language
    config['default']['activecategorie'] = str(activecategorie)
    config['default']['windowxsize'] = str(windowxsize)
    config['default']['windowysize'] = str(windowysize)
    with open(str(userconfigfile), 'w') as configfile:
        config.write(configfile)


def get_tlp_config_file(version: str) -> str:
    if version in ["0_8", "0_9", "1_0", "1_1", "1_2"]:
        return "/etc/default/tlp"

    return "/etc/tlp.conf"


def get_installed_tlp_version() -> str:
    pattern = re.compile(r"TLP ([^\s]+)")
    currentconfig = check_output(["tlp-stat", "-c"]).decode(sys.stdout.encoding)
    matcher = pattern.search(currentconfig)
    version = matcher.group(1).replace(".", "_")
    return version


def get_installed_major_minor_version() -> str:
    return get_installed_tlp_version()[0:3]


if userconfigfile.exists():
    config = configparser.ConfigParser()
    with open(str(userconfigfile)) as configfile:
        config.read_file(configfile)
    try:
        language = config['default']['language']
        activecategorie = int(config['default']['activecategorie'])
        windowxsize = int(config['default']['windowxsize'])
        windowysize = int(config['default']['windowysize'])
    except KeyError:
        # Config key error, override with default values
        persist()
else:
    userconfigpath.mkdir(parents=True, exist_ok=True)
    persist()


# runtime params
tlpbaseversion = get_installed_major_minor_version()
tlpconfigfile = get_tlp_config_file(tlpbaseversion)
tlpconfig = dict()
tlpconfig_original = dict()
tlpconfig_defaults = dict()
imagestate = dict()



from io import open
import os
from json import load


lastvalue = ''
custom_translations = {
    'TLP_PERSISTENT_DEFAULT':       ['1_0', '1_1'],         # ID  / since 1.0
    'CPU_SCALING_GOVERNOR':         ['all'],                # GRP
    'CPU_SCALING_FREQ':             ['all'],                # GRP
    'CPU_HWP':                      ['1_0', '1_1'],         # GRP / since 1.0
    'ENERGY_PERF_POLICY':           ['all'],                # GRP
    'DISK_APM_LEVEL':               ['all'],                # GRP
    'DISK_IOSCHED':                 ['all'],                # ID
    'SATA_LINKPWR':                 ['all'],                # GRP
    'SATA_LINKPWR_BLACKLIST':       ['0_9', '1_0', '1_1'],  # ID  / since 0.9
    'AHCI_RUNTIME_PM':              ['0_9', '1_0', '1_1'],  # GRP / since 0.9
    'AHCI_RUNTIME_PM_TIMEOUT':      ['0_9', '1_0', '1_1'],  # ID  / since 0.9
    'WIFI_PWR':                     ['all'],                # GRP
    'BAY_POWEROFF':                 ['1_0', '1_1'],         # GRP / since 1.0
    'BAY_POWEROFF_ON_BAT':          ['0_8', '0_9'],         # ID  / until 0.9
    'RUNTIME_PM_ALL':               ['0_8', '0_9'],         # ID  / until 0.9
    'RUNTIME_PM_DRIVER_BLACKLIST':  ['all'],                # ID
    'USB_BLACKLIST_BTUSB':          ['1_0', '1_1'],         # ID  / since 1.0
    'USB_BLACKLIST_PHONE':          ['1_0', '1_1'],         # ID  / since 1.0
    'USB_BLACKLIST_PRINTER':        ['1_1'],                # ID  / since 1.1
    'USB_BLACKLIST_WWAN':           ['all'],                # ID
    'RESTORE_THRESHOLDS_ON_BAT':    ['1_0', '1_1']          # ID  / since 1.0
}


def get_json_schema_object_from_file(objectname: str, filename: str) -> dict:
    jsonfile = open(filename)
    jsonobject = load(jsonfile)
    jsonfile.close()
    return jsonobject[objectname]


def add_to_list(listobject: list, value: str):
    global lastvalue
    if not value in listobject:
        if lastvalue in listobject:
            lastindex = listobject.index(lastvalue)
            listobject.insert(lastindex+1, value)
        else:
            listobject.append(value)

    lastvalue = value


def create_translateable_strings_header_file():
    """ This method extracts items for translation from configschema.json and puts them in new file with
    header extension (.h) because poedit and other po-tools do not support JSON files right now. This method
    should be called any time configschema.json changes and translation needs to be updated.
    Change configschema.json -> run this script -> update po files from source with utility (e.g. poedit)"""

    translateobjects = list()

    for file in os.listdir('../configschema'):
        if not file.endswith('.json'):
            continue

        configversion = file.replace('.json', '')
        versionfile = open('configschema.{}.json.h'.format(configversion), 'w+')

        categories = get_json_schema_object_from_file('categories', '../configschema/' + file)
        for category in categories:
            add_to_list(translateobjects, category['name'] + '__CATEGORY_TITLE')
            configs = category['configs']
            for config in configs:
                if 'group' in config:
                    if config['group'] in custom_translations.keys() \
                            and any(v in custom_translations.get(config['group']) for v in {configversion, 'all'}):
                        # versionfile.write('_(\"' + config['group'] + '__GROUP_TITLE' + '\");\n')
                        versionfile.write('_(\"' + config['group'] + '__GROUP_DESCRIPTION' + '\");\n')
                        # configitems = config['ids']
                        # for configitem in configitems:
                        #     versionfile.write('_(\"' + configitem['id'] + '__ID_TITLE' + '\");\n')
                    elif config['group'] not in custom_translations.keys():
                        # add_to_list(translateobjects, config['group'] + '__GROUP_TITLE')
                        add_to_list(translateobjects, config['group'] + '__GROUP_DESCRIPTION')
                        # configitems = config['ids']
                        # for configitem in configitems:
                        #     add_to_list(translateobjects, configitem['id'] + '__ID_TITLE')
                else:
                    if config['id'] in custom_translations.keys() \
                            and any(v in custom_translations.get(config['id']) for v in {configversion, 'all'}):
                        # versionfile.write('_(\"' + config['id'] + '__ID_TITLE' + '\");\n')
                        versionfile.write('_(\"' + config['id'] + '__ID_DESCRIPTION' + '\");\n')
                    elif config['id'] not in custom_translations.keys():
                        # add_to_list(translateobjects, config['id'] + '__ID_TITLE')
                        add_to_list(translateobjects, config['id'] + '__ID_DESCRIPTION')

        versionfile.close()

    newfile = open('configschema.json.h', 'w+')
    for item in translateobjects:
        newfile.write('_(\"' + item + '\");\n')
    newfile.close()


create_translateable_strings_header_file()



import unittest
from importlib import reload

import os
os.environ['XDG_CONFIG_HOME'] = os.path.dirname(os.path.abspath(__file__)) + '/settings/default'
from tlpui import settings
from tlpui.file import read_default_tlp_file_config, get_json_schema_object_from_file


def get_config_count(categories):
    configcount = 0
    for category in categories:
        configs = category['configs']
        for config in configs:
            if 'group' in config:
                groupitems = config['ids']
                configcount += len(groupitems)
            else:
                configcount += 1
    return configcount


class MyTestCase(unittest.TestCase):
    def test_tlp_version_0_8(self):
        reload(settings)
        version = "0_8"
        read_default_tlp_file_config('tlpui/defaults/tlp-{}.conf'.format(version))
        jsoncategories = get_json_schema_object_from_file('categories', 'tlpui/configschema/{}.json'.format(version))
        jsonconfigcount = get_config_count(jsoncategories)

        self.assertEqual(len(jsoncategories), 11)
        self.assertEqual(80, len(settings.tlpconfig_defaults))
        self.assertEqual(jsonconfigcount, len(settings.tlpconfig_defaults))

    def test_tlp_version_0_9(self):
        reload(settings)
        version = "0_9"
        read_default_tlp_file_config('tlpui/defaults/tlp-{}.conf'.format(version))
        jsoncategories = get_json_schema_object_from_file('categories', 'tlpui/configschema/{}.json'.format(version))
        jsonconfigcount = get_config_count(jsoncategories)

        self.assertEqual(len(jsoncategories), 11)
        self.assertEqual(84, len(settings.tlpconfig_defaults))
        self.assertEqual(jsonconfigcount, len(settings.tlpconfig_defaults))

    def test_tlp_version_1_0(self):
        reload(settings)
        version = "1_0"
        read_default_tlp_file_config('tlpui/defaults/tlp-{}.conf'.format(version))
        jsoncategories = get_json_schema_object_from_file('categories', 'tlpui/configschema/{}.json'.format(version))
        jsonconfigcount = get_config_count(jsoncategories)

        self.assertEqual(len(jsoncategories), 11)
        self.assertEqual(90, len(settings.tlpconfig_defaults))
        self.assertEqual(jsonconfigcount, len(settings.tlpconfig_defaults))

    def test_tlp_version_1_1(self):
        reload(settings)
        version = "1_1"
        read_default_tlp_file_config('tlpui/defaults/tlp-{}.conf'.format(version))
        jsoncategories = get_json_schema_object_from_file('categories', 'tlpui/configschema/{}.json'.format(version))
        jsonconfigcount = get_config_count(jsoncategories)

        self.assertEqual(len(jsoncategories), 11)
        self.assertEqual(91, len(settings.tlpconfig_defaults))
        self.assertEqual(jsonconfigcount, len(settings.tlpconfig_defaults))

    def test_tlp_version_1_2(self):
        reload(settings)
        version = "1_2"
        read_default_tlp_file_config('tlpui/defaults/tlp-{}.conf'.format(version))
        jsoncategories = get_json_schema_object_from_file('categories', 'tlpui/configschema/{}.json'.format(version))
        jsonconfigcount = get_config_count(jsoncategories)

        self.assertEqual(len(jsoncategories), 11)
        self.assertEqual(100, len(settings.tlpconfig_defaults))
        self.assertEqual(jsonconfigcount, len(settings.tlpconfig_defaults))

    def test_tlp_version_1_3(self):
        reload(settings)
        version = "1_3"
        read_default_tlp_file_config('tlpui/defaults/tlp-{}.conf'.format(version))
        jsoncategories = get_json_schema_object_from_file('categories', 'tlpui/configschema/{}.json'.format(version))
        jsonconfigcount = get_config_count(jsoncategories)

        self.assertEqual(len(jsoncategories), 11)
        self.assertEqual(99, len(settings.tlpconfig_defaults))
        self.assertEqual(jsonconfigcount, len(settings.tlpconfig_defaults))


if __name__ == '__main__':
    unittest.main()



from . import settings
import gettext


def load_lang(langfile):
    translation = gettext.translation(domain=langfile, localedir=settings.langdir, languages=[settings.language])

    versionlangfile = "{}{}".format(langfile, settings.tlpbaseversion)
    if gettext.find(domain=versionlangfile, localedir=settings.langdir, languages=[settings.language]) is None:
        return translation.gettext
    else:
        versiontranslation = gettext.translation(domain=versionlangfile, localedir=settings.langdir, languages=[settings.language])
        versiontranslation.add_fallback(translation)
        return versiontranslation.gettext


CDT_ = load_lang('configdescriptions')
MT_ = load_lang('mainui')
ST_ = load_lang('statui')
UH_ = load_lang('uihelper')



from gi.repository import Gtk
from .. import settings


def create_selection_box(configname: str, values: str) -> Gtk.ComboBox:
    combobox = Gtk.ComboBoxText()
    selectitems = values.split(',')
    configvalue = settings.tlpconfig[configname].get_value()

    countid = 0
    selectid = 0

    for item in selectitems:
        combobox.append_text(item)
        if item == configvalue:
            selectid = countid
        countid += 1

    combobox.set_active(selectid)
    combobox.connect('changed', change_selection_state, configname)
    return combobox


def change_selection_state(self: Gtk.ComboBoxText, configname: str):
    newvalue = self.get_active_text()
    settings.tlpconfig[configname].set_value(newvalue)



from gi.repository import Gtk
from .. import settings


def create_multi_selection_box(configname: str, values: str) -> Gtk.ComboBox:
    multiselectbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
    multiselectitems = values.split(',')
    configitems = settings.tlpconfig[configname].get_value().split(' ')

    for selectitem in multiselectitems:
        toggle = Gtk.ToggleButton(selectitem)
        if selectitem in configitems:
            toggle.set_active(True)
        toggle.connect('toggled', change_selection_state, configname, multiselectitems)

        multiselectbox.pack_start(toggle, False, False, 2)

    return multiselectbox


def change_selection_state(self: Gtk.ToggleButton, configname: str, checkitems: []):
    currentitem = self.get_label()
    currentstate = self.get_active()
    currentvalue = str(settings.tlpconfig[configname].get_value())

    newvalue = ''
    for checkitem in checkitems:
        if checkitem == currentitem and currentstate is False:
            continue
        elif checkitem in currentvalue or checkitem == currentitem:
            newvalue = newvalue + " " + checkitem

    settings.tlpconfig[configname].set_value(newvalue.lstrip())



from enum import Enum
from . import settings


class ConfType(Enum):
    DEFAULT = 1
    DROPIN = 2
    USER = 3
    ERR = 4


class TlpDefaults:
    def __init__(self, name: str, value: str, enabled: bool):
        self.name = name
        self.value = value
        self.enabled = enabled

    def get_name(self) -> str:
        return self.name

    def get_value(self) -> str:
        return self.value

    def is_enabled(self) -> bool:
        return self.enabled


class TlpConfig:
    def __init__(self, enabled: bool, name: str, value: str, conftype: ConfType, confpath=""):
        self.enabled = enabled
        self.enabledstore = enabled
        self.name = name
        self.value = value
        self.valuestore = value
        self.conftype = conftype
        self.confpath = confpath

    def get_name(self) -> str:
        return self.name

    def get_conf_type(self) -> ConfType:
        return self.conftype

    def get_conf_path(self) -> str:
        return self.confpath

    def get_value(self) -> str:
        return self.value

    def set_value(self, newvalue: str):
        self.value = newvalue
        self.refresh_image_state()

    def set_enabled(self, newstate: bool):
        self.enabled = newstate
        self.refresh_image_state()

    def is_enabled(self) -> bool:
        return self.enabled

    def refresh_image_state(self):
        settings.imagestate[self.name].refresh_image_state(self.value, self.valuestore, self.enabled, self.enabledstore)


def get_changed_properties() -> dict:
    changedproperties = dict()

    changed = settings.tlpconfig
    original = settings.tlpconfig_original

    for configid in changed:
        config = changed[configid]              # type: TlpConfig
        config_original = original[configid]    # type: TlpConfig

        statechange = config.is_enabled() != config_original.is_enabled()
        configchange = config.get_value() != config_original.get_value()

        if statechange or configchange:
            configname = config.get_name()

            if not config.is_enabled() and settings.tlpconfig_defaults[configname].is_enabled():
                enabled = ""
                value = "* empty"
            else:
                enabled = "" if config.is_enabled() else "#"
                value = config.get_value()

            value = '\"' + value + '\"'
            changedproperties[configname] = "{}{}={}".format(enabled, configname, value)

    return changedproperties


import copy
import re
from sys import stdout
from subprocess import check_output, STDOUT
from io import open
from json import load
from os import access, W_OK, close, path
from tempfile import mkstemp
from .config import TlpDefaults, TlpConfig, ConfType
from . import settings
from .uihelper import get_graphical_sudo, SUDO_MISSING_TEXT


def get_json_schema_object(objectname) -> dict:
    tlpprovidedschema = '/usr/share/tlp-pm/configschema.json'
    if path.exists(tlpprovidedschema):
        return get_json_schema_object_from_file(objectname, tlpprovidedschema)
    else:
        majorminor = settings.tlpbaseversion
        return get_json_schema_object_from_file(objectname, settings.workdir + '/configschema/' + majorminor + '.json')


def get_json_schema_object_from_file(objectname: str, filename: str) -> dict:
    jsonfile = open(filename)
    jsonobject = load(jsonfile)
    jsonfile.close()
    return jsonobject[objectname]


def init_tlp_file_config() -> None:
    tlpversion = settings.tlpbaseversion
    read_default_tlp_file_config('{}/defaults/tlp-{}.conf'.format(settings.workdir, tlpversion))

    if tlpversion not in ["0_8", "0_9", "1_0", "1_1", "1_2"]:
        # update default values with intrinsic ones
        intrinsic_defaults_path = "/usr/share/tlp/defaults.conf"
        read_default_tlp_file_config(intrinsic_defaults_path)

    # get current settings from tlp itself
    simple_stat_command = ["tlp-stat", "-c"]
    tlpstat = check_output(simple_stat_command, stderr=STDOUT).decode(stdout.encoding)
    tlpsettinglines = tlpstat.split('\n')

    if tlpversion in ["0_8", "0_9", "1_0", "1_1", "1_2"]:
        extract_tlp_settings_obsolete(tlpsettinglines)
    else:
        extract_tlp_settings(tlpsettinglines)

    # Add default tlp configs not set in current settings
    for key, default in settings.tlpconfig_defaults.items():     # type: TlpDefaults
        configname = default.get_name()
        if configname not in settings.tlpconfig.keys():
            enabled = default.is_enabled()
            value = default.get_value()
            settings.tlpconfig[configname] = TlpConfig(enabled, configname, value, ConfType.DEFAULT, "")

    # finally store copy for comparing changes
    settings.tlpconfig_original = copy.deepcopy(settings.tlpconfig)


def extract_tlp_settings_obsolete(lines: list) -> None:
    propertypattern = re.compile(r'^[A-Z_\d]+=')

    for line in lines:
        if propertypattern.match(line):
            cleanline = line.lstrip().rstrip()

            property = cleanline.split('=', maxsplit=1)
            propertyname = property[0]
            propertyvalue = property[1]

            if propertyvalue.startswith('\"') and propertyvalue.endswith('\"'):
                propertyvalue = propertyvalue.lstrip('\"').rstrip('\"')

            settings.tlpconfig[propertyname] = TlpConfig(True, propertyname, propertyvalue, ConfType.USER, settings.tlpconfigfile)


def extract_tlp_settings(lines: list) -> None:
    propertypattern = re.compile(r'^.+?\.conf\sL\d+\:\s[A-Z_\d]+=')

    for line in lines:
        if propertypattern.match(line):
            cleanline = line.lstrip().rstrip()
            settingsparts = cleanline.split(' ', maxsplit=2)
            configfile = settingsparts[0]       # type: str
            if configfile == 'defaults.conf':
                conftype = ConfType.DEFAULT
            elif configfile.startswith('/etc/tlp.d/'):
                conftype = ConfType.DROPIN
            elif configfile == "/etc/tlp.conf":
                conftype = ConfType.USER
            else:
                print('Config type not found for file: {}'.format(configfile))
                conftype = ConfType.ERR

            property = settingsparts[2].split('=', maxsplit=1)
            propertyname = property[0]
            propertyvalue = property[1]

            if propertyvalue.startswith('\"') and propertyvalue.endswith('\"'):
                propertyvalue = propertyvalue.lstrip('\"').rstrip('\"')

            settings.tlpconfig[propertyname] = TlpConfig(True, propertyname, propertyvalue, conftype, configfile)


def read_default_tlp_file_config(filename: str) -> None:
    propertypattern = re.compile(r'^#?[A-Z_\d]+=')
    fileopener = open(filename)
    lines = fileopener.readlines()
    fileopener.close()

    for line in lines:
        if propertypattern.match(line):
            cleanline = line.lstrip().rstrip()

            if (cleanline.startswith('#')):
                enabled = False
                cleanline = cleanline.lstrip('#')
            else:
                enabled = True

            property = cleanline.split('=', maxsplit=1)
            propertyname = property[0]
            propertyvalue = property[1]

            if propertyvalue.startswith('\"') and propertyvalue.endswith('\"'):
                propertyvalue = propertyvalue.lstrip('\"').rstrip('\"')

            settings.tlpconfig_defaults[propertyname] = TlpDefaults(propertyname, propertyvalue, enabled)


def create_tmp_tlp_config_file(changedproperties: dict) -> str:
    propertypattern = re.compile(r'^#?[A-Z_\d]+=')
    fh, tmpfilename = mkstemp()
    newfile = open(tmpfilename, 'w')

    oldfile = open(settings.tlpconfigfile)
    lines = oldfile.readlines()
    oldfile.close()

    for line in lines:
        if propertypattern.match(line):
            cleanline = line.lstrip().lstrip('#')

            property = cleanline.split('=', maxsplit=1)
            propertyname = property[0]

            if propertyname in changedproperties.keys():
                if cleanline.startswith(propertyname + "="):
                    newfile.write(changedproperties[propertyname] + '\n')
                    continue

        newfile.write(line)

    newfile.close()
    close(fh)
    oldfile.close()

    return tmpfilename


def write_tlp_config(tmpconfigfile: str) -> str:
    sedtlpconfigfile = "w" + settings.tlpconfigfile
    sedcommand = ["sed", "-n", sedtlpconfigfile, tmpconfigfile]

    # check permission and apply sudo if needed
    if not access(settings.tlpconfigfile, W_OK):
        sudo_cmd = get_graphical_sudo()
        if sudo_cmd is None:
            return SUDO_MISSING_TEXT
        sedcommand.insert(0, sudo_cmd)

    check_output(sedcommand)
    return ''



from gi.repository import Gtk
from .. import settings

opacitylevel = 0.3

def create_toggle_button(configname:str, tlpconfigbox: Gtk.Box) -> Gtk.CheckButton:
    togglebutton = Gtk.CheckButton()

    if settings.tlpconfig[configname].is_enabled():
        togglebutton.set_active(True)
    else:
        tlpconfigbox.set_opacity(opacitylevel)

    togglebutton.connect('toggled', on_button_toggled, configname, tlpconfigbox)
    return togglebutton


def on_button_toggled(self: Gtk.CheckButton, configname: str, tlpconfigbox: Gtk.Box):
    tlpobject = settings.tlpconfig[configname]

    if self.get_active():
        tlpobject.set_enabled(True)
        tlpconfigbox.set_opacity(1)
    else:
        tlpobject.set_enabled(False)
        tlpconfigbox.set_opacity(opacitylevel)



"""This module provides the main window parts for TLPUI"""

import importlib
import difflib

from pathlib import Path
from gi.repository import Gtk, Gdk
from . import settings
from . import language
from .config import get_changed_properties
from .configui import create_config_box
from .file import init_tlp_file_config, create_tmp_tlp_config_file, write_tlp_config
from .statui import create_stat_box
from .uihelper import get_theme_image


def store_window_size(self) -> None:
    """Store current window size in settings"""
    settings.windowxsize = self.get_size()[0]
    settings.windowysize = self.get_size()[1]


def window_key_events(self, event) -> None:
    """Add window key events like crtl+(q|w|s)"""
    if event.state & Gdk.ModifierType.CONTROL_MASK:
        # close window with ctrl+q or ctrl+w
        if event.keyval == 113 or event.keyval == 119:
            quit_tlp_config(None, self)
        # save config with ctrl+s
        if event.keyval == 115:
            save_tlp_config(None, self)


def close_main_window(self, _) -> bool:
    """Close main window"""
    quit_tlp_config(None, self)

    # When delete-event is cancelled we have to return True
    # Otherwise application window will disappear
    return True


def load_tlp_config(_, window: Gtk.Window, reloadtlpconfig: bool) -> None:
    """Load TLP configuration to UI"""

    if reloadtlpconfig:
        init_tlp_file_config()

    newmainbox = create_main_box(window)
    children = window.get_children()
    for child in children:
        window.remove(child)
    window.add(newmainbox)
    window.show_all()


def save_tlp_config(self, window) -> None:
    """Persists TLP configuration changes"""
    changedproperties = get_changed_properties()
    if len(changedproperties) == 0:
        return

    tmpfilename = create_tmp_tlp_config_file(changedproperties)

    saveresponse = changed_items_dialog(
        window,
        tmpfilename,
        language.MT_('Review settings'),
        language.MT_('Save these changes?'))

    if saveresponse == Gtk.ResponseType.OK:

        output = write_tlp_config(tmpfilename)
        if output != '':
            dialog = Gtk.MessageDialog(window)
            dialog.set_default_size(150, 100)
            dialog.format_secondary_markup(output)
            dialog.run()
            dialog.destroy()
            return

        # reload config after file save
        load_tlp_config(self, window, True)


def quit_tlp_config(_, window) -> None:
    """Quit TLPUI and prompt for unsaved changes"""
    settings.persist()

    changedproperties = get_changed_properties()
    if len(changedproperties) == 0:
        Gtk.main_quit()
        return

    tmpfilename = create_tmp_tlp_config_file(changedproperties)

    quitresponse = changed_items_dialog(
        window,
        tmpfilename,
        language.MT_('Unsaved settings'),
        language.MT_('Do you really want to quit? No changes will be saved.'))

    if quitresponse == Gtk.ResponseType.OK:
        Gtk.main_quit()


def changed_items_dialog(window, tmpfilename: str, dialogtitle: str, message: str) -> Gtk.ResponseType:
    """Dialog to show changed TLP configuration items"""
    dialog = Gtk.Dialog(dialogtitle, window, 0, (
        Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
        Gtk.STOCK_OK, Gtk.ResponseType.OK
    ))
    dialog.set_default_size(400, 300)

    scrolledwindow = Gtk.ScrolledWindow()
    scrolledwindow.set_hexpand(True)
    scrolledwindow.set_vexpand(True)

    fromfilecontent = open(settings.tlpconfigfile, 'r').readlines()
    tofilecontent = open(tmpfilename, 'r').readlines()
    diff = settings.tlpconfigfile + '\n\n'
    for line in difflib.unified_diff(fromfilecontent, tofilecontent, n=0, lineterm=''):
        if line.startswith('---') or line.startswith('+++'):
            continue
        postfix = '' if line.startswith('-') else '\n'
        diff += line + postfix

    textbuffer = Gtk.TextBuffer()
    textbuffer.set_text(diff)

    textview = Gtk.TextView()
    textview.set_buffer(textbuffer)
    textview.set_editable(False)

    scrolledwindow.add(textview)
    scrolledwindow.set_border_width(12)

    box = dialog.get_content_area()
    box.pack_start(scrolledwindow, True, True, 0)
    box.pack_start(Gtk.Label('\n{}\n'.format(message)), False, False, 0)

    dialog.show_all()
    response = dialog.run()
    dialog.destroy()

    return response


def create_menu_box(window, fileentry) -> Gtk.Box:
    """Create application menu from XML structure"""
    xmlmenustructure = """
    <ui>
        <menubar name='MenuBar'>
            <menu action='FileMenu'>
                <menuitem action='save' />
                <menuitem action='quit' />
            </menu>
            <menu action='LanguageMenu'>
                <menuitem name="en_EN" action='en_EN' />
                <menuitem name="de_DE" action='de_DE' />
                <menuitem name="ru_RU" action='ru_RU' />
                <menuitem name="id_ID" action='id_ID' />
            </menu>
        </menubar>
    </ui>
    """

    uimanager = Gtk.UIManager()
    uimanager.add_ui_from_string(xmlmenustructure)

    actiongroup = Gtk.ActionGroup("actions")
    add_menu_actions(window, fileentry, actiongroup)
    uimanager.insert_action_group(actiongroup)

    menubar = uimanager.get_widget("/MenuBar")

    menubox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
    menubox.pack_start(menubar, False, False, 0)

    return menubox


def create_settings_box(window, fileentry) -> Gtk.Box:
    """Buttons for direct access in UI"""
    reloadbutton = Gtk.Button(label=' ' + language.MT_('Reload'), image=get_theme_image('view-refresh-symbolic', Gtk.IconSize.BUTTON))
    reloadbutton.connect('clicked', load_tlp_config, window, True)
    reloadbutton.set_always_show_image(True)
    savebutton = Gtk.Button(label=' ' + language.MT_('Save'), image=get_theme_image('document-save-symbolic', Gtk.IconSize.BUTTON))
    savebutton.connect('clicked', save_tlp_config, window)
    savebutton.set_always_show_image(True)

    settingsbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=12)
    settingsbox.pack_start(fileentry, True, True, 0)
    settingsbox.pack_start(reloadbutton, False, False, 0)
    settingsbox.pack_start(savebutton, False, False, 0)

    return settingsbox


def add_menu_actions(window, fileentry, actiongroup) -> None:
    """Add actions to application menu"""
    actionfilemenu = Gtk.Action("FileMenu", language.MT_("File"), None, None)
    actiongroup.add_action(actionfilemenu)
    actionfilemenusave = Gtk.Action("save", language.MT_('Save'), None, Gtk.STOCK_SAVE)
    actionfilemenusave.connect("activate", save_tlp_config, window)
    actiongroup.add_action(actionfilemenusave)
    actionfilemenuquit = Gtk.Action("quit", language.MT_('Quit'), None, Gtk.STOCK_QUIT)
    actionfilemenuquit.connect("activate", quit_tlp_config, window)
    actiongroup.add_action(actionfilemenuquit)

    actionlanguagemenu = Gtk.Action("LanguageMenu", language.MT_("Language"), None, None)
    actiongroup.add_action(actionlanguagemenu)

    langdir = Path(settings.langdir)
    for langobject in langdir.iterdir():
        if langobject.is_dir():
            lang = langobject.name

            if lang == settings.language:
                actionlang = Gtk.Action(lang, lang, None, Gtk.STOCK_APPLY)
            else:
                actionlang = Gtk.Action(lang, lang, None, None)

            actionlang.connect("activate", switch_language, lang, window)
            actiongroup.add_action(actionlang)


def switch_language(self, lang: str, window: Gtk.Window) -> None:
    """Language switcher"""
    settings.language = lang

    # reload language values
    importlib.reload(language)

    load_tlp_config(self, window, False)


def create_main_box(window: Gtk.Window) -> Gtk.Box:
    """Create TLP configuration items notebook view"""
    notebook = Gtk.Notebook()
    notebook.set_tab_pos(Gtk.PositionType.TOP)

    fileentry = Gtk.Label(settings.tlpconfigfile)
    fileentry.set_alignment(0, 0.5)

    menubox = create_menu_box(window, fileentry)
    settingsbox = create_settings_box(window, fileentry)
    configbox = create_config_box(window)

    contentbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=12)
    contentbox.set_margin_top(18)
    contentbox.set_margin_bottom(18)
    contentbox.set_margin_left(18)
    contentbox.set_margin_right(18)
    contentbox.pack_start(settingsbox, False, False, 0)
    contentbox.pack_start(configbox, True, True, 0)

    configlabel = Gtk.Label(language.MT_('Configuration'))
    configlabel.set_hexpand(True)

    statlabel = Gtk.Label(language.MT_('Statistics'))
    statlabel.set_hexpand(True)

    statbox = create_stat_box()

    notebook.append_page(contentbox, configlabel)
    notebook.append_page(statbox, statlabel)

    mainbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
    mainbox.pack_start(menubox, False, False, 0)
    mainbox.pack_start(notebook, True, True, 0)

    return mainbox



from gi.repository import Gtk
from .. import settings


def create_checkbutton_box(configname: str, values: str) -> Gtk.Box:
    checkbox = Gtk.Box()
    checkitems = values.split(',')
    configvalue = settings.tlpconfig[configname].get_value()

    checkbuttonitem = 0
    for checkitem in checkitems:
        checkbutton = Gtk.CheckButton(checkitem)
        if checkitem in configvalue:
            checkbutton.set_active(True)
        checkbutton.connect('toggled', change_check_state, configname, checkitems)

        if checkbuttonitem % 2 == 0:
            checkbox.pack_start(checkbutton, False, False, 0)
        else:
            checkbox.pack_start(checkbutton, False, False, 12)
        checkbuttonitem += 1

    return checkbox


def change_check_state(self: Gtk.CheckButton, configname: str, checkitems: []):
    currentitem = self.get_label()
    currentstate = self.get_active()
    currentvalue = str(settings.tlpconfig[configname].get_value())

    newvalue = ''
    for checkitem in checkitems:
        if checkitem == currentitem and currentstate is False:
            continue
        elif checkitem in currentvalue or checkitem == currentitem:
            newvalue = newvalue + " " + checkitem

    settings.tlpconfig[configname].set_value(newvalue.lstrip())



from gi.repository import Gtk
from .. import settings


def create_numeric_spinbutton(configname: str, values: str) -> Gtk.SpinButton:
    range = values.split('-')
    configvalue = settings.tlpconfig[configname].get_value()
    adjustment = Gtk.Adjustment(0, float(range[0]), float(range[1]), 1, 10, 0)

    spinbutton = Gtk.SpinButton()
    spinbutton.set_numeric(True)
    spinbutton.set_adjustment(adjustment)
    spinbutton.set_value(float(configvalue))
    spinbutton.connect('value-changed', change_numeric_spin_value, configname)
    return spinbutton


def change_numeric_spin_value(self: Gtk.SpinButton, configname: str):
    newvalue = str(int(self.get_value()))
    settings.tlpconfig[configname].set_value(newvalue)



import sys, re
from gi.repository import Gtk

from collections import OrderedDict
from subprocess import check_output
from .. import settings
from ..uihelper import get_theme_image

global indexstore

def create_list(configname: str, window: Gtk.Window) -> Gtk.Box:
    box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
    label = Gtk.Label(settings.tlpconfig[configname].get_value().replace(" ", "\n"))

    button = Gtk.Button(label=' Edit', image=get_theme_image('edit-symbolic', Gtk.IconSize.BUTTON))
    button.connect('clicked', edit_list, configname, label, window)
    button.set_always_show_image(True)

    box.pack_start(label, False, False, 0)
    box.pack_start(button, False, False, 12)
    return box


def edit_list(self, configname: str, usblistlabel: Gtk.Label, window: Gtk.Window):
    tlpobject = settings.tlpconfig[configname]
    usblistpattern = re.compile(r'^.+?([a-f\d]{4}:[a-f\d]{4})(.+?)$')
    currentitems = OrderedDict()
    if tlpobject.get_value() != '':
        for item in tlpobject.get_value().split(' '):
            currentitems[item] = ["", True]

    tlpusblist = check_output(["tlp-usblist"]).decode(sys.stdout.encoding)

    usbitems = OrderedDict()
    for line in tlpusblist.splitlines():
        matcher = usblistpattern.match(line)
        usbid = matcher.group(1)
        description = matcher.group(2)
        active = False

        # only add usb id once
        if usbid in usbitems:
            continue

        # check if item is selected
        if usbid in currentitems:
            active = True
            del currentitems[usbid]
        usbitems[usbid] = [description, active]

    usbitems.update(currentitems)

    grid = Gtk.Grid()
    grid.set_row_homogeneous(True)
    grid.set_column_spacing(12)

    grid.attach(Gtk.Label(''), 0, 0, 1, 1)
    grid.attach(Gtk.Label(label='ID', halign=Gtk.Align.START), 1, 0, 1, 1)
    grid.attach(Gtk.Label(label='Description', halign=Gtk.Align.START), 2, 0, 1, 1)
    grid.attach(Gtk.Label(''), 3, 0, 1, 1)

    rowindex = 2
    allitems = list()
    selecteditems = list()
    for key, value in usbitems.items():
        allitems.append(key)
        toggle = Gtk.ToggleButton(key)
        toggle.connect('toggled', on_button_toggled, key, selecteditems)

        if value[1]:
            toggle.set_active(True)

        label = Gtk.Label(value[0])
        label.set_halign(Gtk.Align.START)

        grid.attach(toggle, 1, rowindex, 1, 1)
        grid.attach(label, 2, rowindex, 1, 1)

        rowindex += 1

    addbutton = Gtk.Button(label=' Add', image=get_theme_image('list-add-symbolic', Gtk.IconSize.BUTTON))
    addbutton.set_sensitive(False)
    addbutton.set_always_show_image(True)

    addentry = Gtk.Entry()
    addentry.set_width_chars(9)
    addentry.set_max_length(9)
    addentry.connect('changed', usb_entry_check, addbutton)

    addbutton.connect('clicked', add_usb_item, addentry, grid, allitems, selecteditems)

    addlabel = Gtk.Label(label='Add custom unattached USB IDs:', halign=Gtk.Align.START)

    addbox = Gtk.Box()
    addbox.pack_start(addentry, False, False, 0)
    addbox.pack_start(addbutton, False, False, 12)
    addbox.pack_start(Gtk.Label(''), True, True, 0)

    grid.attach(addlabel, 1, rowindex, 2, 1)
    grid.attach(addbox, 1, rowindex+1, 2, 1)

    global indexstore
    indexstore = rowindex+2

    dialog = Gtk.Dialog('Usb devices', window, 0, (
        Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
        Gtk.STOCK_OK, Gtk.ResponseType.OK
    ))

    contentarea = dialog.get_content_area()
    contentarea.set_spacing(6)
    contentarea.add(grid)
    dialog.show_all()

    response = dialog.run()
    if response == Gtk.ResponseType.OK:
        configvalue = ' '.join(str(item) for item in selecteditems)
        tlpobject.set_value(configvalue)
        usblistlabel.set_text(configvalue.replace(" ", "\n"))

    dialog.destroy()


def on_button_toggled(self: Gtk.ToggleButton, key: str, selecteditems: list):
    if self.get_active():
        selecteditems.append(key)
    else:
        selecteditems.remove(key)


def usb_entry_check(self: Gtk.Entry, button: Gtk.Button):
    usbpattern = re.compile(r'^[a-f\d]{4}:[a-f\d]{4}$')
    if usbpattern.match(self.get_text()):
        self.set_name('validEntry')
        button.set_sensitive(True)
    else:
        self.set_name('invalidEntry')
        button.set_sensitive(False)


def add_usb_item(self, entry: Gtk.Entry, grid: Gtk.Grid, allitems: list, selecteditems: list):
    key = entry.get_text()
    if key in allitems:
        return
    else:
        allitems.append(key)

    toggle = Gtk.ToggleButton(key)
    toggle.connect('toggled', on_button_toggled, key, selecteditems)
    toggle.set_active(True)

    label = Gtk.Label('')
    label.set_halign(Gtk.Align.START)

    global indexstore
    grid.attach(toggle, 1, indexstore, 1, 1)
    grid.attach(label, 2, indexstore, 1, 1)

    indexstore += 1
    grid.show_all()
